@@ -17,8 +17,8 @@
 --]]
 
 -- Version information: should be identical to that in l3build.dtx
-release_date = "2016/01/19"
-release_ver  = "6377"
+release_date = "2016/11/21"
+release_ver  = "6760"
 
 -- "module" is a deprecated function in Lua 5.2: as we want the name
 -- for other purposes, and it should eventually be 'free', simply
@@ -110,7 +110,7 @@ typesetexe = typesetexe or "pdflatex"
 unpackexe  = unpackexe  or "tex"
 zipexe     = "zip"
 
-checkopts   = checkopts   or "-interaction=batchmode"
+checkopts   = checkopts   or "-interaction=nonstopmode"
 cmdchkopts  = cmdchkopts  or "-interaction=batchmode"
 typesetopts = typesetopts or "-interaction=nonstopmode"
 unpackopts  = unpackopts  or ""
@@ -149,6 +149,7 @@ makeindexopts = makeindexopts or ""
 -- Other required settings
 asciiengines = asciiengines or {"pdftex"}
 checkruns    = checkruns    or 1
+epoch        = epoch        or 1463734800
 maxprintline = maxprintline or 79
 packtdszip   = packtdszip   or false -- Not actually needed but clearer
 scriptname   = scriptname   or "build.lua" -- Script used in each directory
@@ -157,9 +158,12 @@ versionform  = versionform  or ""
 
 -- Extensions for various file types: used to abstract out stuff a bit
 bakext = bakext or ".bak"
+dviext = dviext or ".dvi"
 logext = logext or ".log"
 lveext = lveext or ".lve"
 lvtext = lvtext or ".lvt"
+pdfext = pdfext or ".pdf"
+psext  = psext  or ".ps"
 tlgext = tlgext or ".tlg"
 
 -- Run time options
@@ -176,8 +180,9 @@ function argparse()
       ["halt-on-error"]   = "halt"   ,
       ["halt-on-failure"] = "halt"   ,
       help                = "help"   ,
+      pdf                 = "pdf" ,
       quiet               = "quiet"  ,
-      version             = "version"
+      release             = "release"
     }
   local short_options =
     {
@@ -185,8 +190,9 @@ function argparse()
       e = "engine" ,
       h = "help"   ,
       H = "halt"   ,
+      p = "pdf"  ,
       q = "quiet"  ,
-      v = "version"
+      r = "release"
     }
   local option_args =
     {
@@ -194,8 +200,9 @@ function argparse()
       engine  = true ,
       halt    = false,
       help    = false,
+      pdf     = false,
       quiet   = false,
-      version = true
+      release = true
     }
   -- arg[1] is a special case: must be a command or "-h"/"--help"
   -- Deal with this by assuming help and storing only apparently-valid
@@ -303,8 +310,9 @@ optdate    = userargs["date"]
 optengines = userargs["engine"]
 opthalt    = userargs["halt"]
 opthelp    = userargs["help"]
+optpdf     = userargs["pdf"]
 optquiet   = userargs["quiet"]
-optversion = userargs["version"]
+optrelease = userargs["release"]
 
 -- Convert a file glob into a pattern for use by e.g. string.gub
 -- Based on https://github.com/davidm/lua-glob-pattern
@@ -387,6 +395,8 @@ end
 -- the support functions
 if os.type == "windows" then
   os_ascii    = "@echo."
+  os_cmpexe   = os.getenv("cmpexe") or "fc /b"
+  os_cmpext   = os.getenv("cmpext") or ".cmp"
   os_concat   = "&"
   os_diffext  = os.getenv("diffext") or ".fc"
   os_diffexe  = os.getenv("diffexe") or "fc /n"
@@ -399,6 +409,8 @@ if os.type == "windows" then
   os_yes      = "for /l %I in (1,1,200) do @echo y"
 else
   os_ascii    = "echo \"\""
+  os_cmpexe   = os.getenv("cmpexe") or "cmp"
+  os_cmpext   = os.getenv("cmpext") or ".cmp"
   os_concat   = ";"
   os_diffext  = os.getenv("diffext") or ".diff"
   os_diffexe  = os.getenv("diffexe") or "diff -c --strip-trailing-cr"
@@ -597,9 +609,9 @@ function allmodules(target)
     if optengines then
       engines = " --engine=" .. table.concat(optengines, ",")
     end
-    local version = ""
-    if optversion then
-      version = " --version=" .. optversion[1]
+    local release = ""
+    if optrelease then
+      release = " --release=" .. optrelease[1]
     end
     local errorlevel = run(
       i,
@@ -607,7 +619,9 @@ function allmodules(target)
         .. (opthalt and " -H" or "")
         .. date
         .. engines
-        .. version
+        .. (optpdf and " -p" or "")
+        .. (optquiet and " -q" or "")
+        .. release
     )
     if errorlevel ~= 0 then
       return errorlevel
@@ -747,7 +761,7 @@ function formatlog(logfile, newfile, engine)
     return false
   end
     -- Substitutions to remove some non-useful changes
-  local function normalize(line)
+  local function normalize(line, lastline)
     -- Zap line numbers from \show, \showbox, \box_show and the like:
     -- do this before wrapping lines
     line = string.gsub(line, "^l%.%d+ ", "l. ...")
@@ -755,8 +769,7 @@ function formatlog(logfile, newfile, engine)
     -- Skip lines that have an explicit marker for truncation
     if string.len(line) == maxprintline  and
        not string.match(line, "%.%.%.$") then
-      lastline = (lastline or "") .. line
-      return ""
+      return "", (lastline or "") .. line
     end
     local line = (lastline or "") .. line
     lastline = ""
@@ -779,14 +792,12 @@ function formatlog(logfile, newfile, engine)
     if checksearch then
       line = string.gsub(line, "%(.*/([%w-]+%.[%w-]+)%)?%s*$", "(../%1")
     end
-    -- Zap map loading of map
-    line = string.gsub(line, "%{%w?:?[%w/%-]*/pdftex%.map%}", "")
     -- Deal with the fact that "(.aux)" may have still a leading space
     line = string.gsub(line, "^ %(%.aux%)", "(.aux)")
     -- Merge all of .fd data into one line so will be removed later
     if string.match(line, "^ *%([%.%/%w]+%.fd[^%)]*$") then
       lastline = (lastline or "") .. line
-      return ""
+      return "", (lastline or "") .. line
     end
     -- TeX90/XeTeX knows only the smaller set of dimension units
     line = string.gsub(
@@ -805,6 +816,25 @@ function formatlog(logfile, newfile, engine)
     end
     -- Remove 'normal' direction information on boxes with (u)pTeX
     line = string.gsub(line, ",? yoko direction,?", "")
+    -- Remove the \special line that in DVI mode keeps PDFs comparable
+    if string.match(line, "^%.*\\special%{pdf: docinfo << /Creator") then
+      return ""
+    end
+    -- Remove the \special line possibly present in DVI mode for paper size
+    if string.match(line, "^%.*\\special%{papersize") then
+      return ""
+    end
+    -- Remove ConTeXt stuff
+    if string.match(line, "^backend         >") or
+       string.match(line, "^close source    >") or
+       string.match(line, "^mkiv lua stats  >") or
+       string.match(line, "^pages           >") or
+       string.match(line, "^system          >") or
+       string.match(line, "^used file       >") or
+       string.match(line, "^used option     >") or
+       string.match(line, "^used structure  >") then
+       return ""
+    end
     -- A tidy-up to keep LuaTeX and other engines in sync
     local utf8_char = unicode.utf8.char
     line = string.gsub(line, utf8_char(127), "^^?")
@@ -815,7 +845,7 @@ function formatlog(logfile, newfile, engine)
         line = string.gsub(line, utf8_char(i), "^^" .. string.format("%02x", i))
       end
     end
-    return line
+    return line, lastline
   end
   local lastline = ""
   local newlog = ""
@@ -828,10 +858,10 @@ function formatlog(logfile, newfile, engine)
       break
     elseif line == "OMIT" then
       skipping = true
-    elseif line == "TIMO" then
+    elseif string.match(line, "^%)?TIMO$") then
       skipping = false
     elseif not prestart and not skipping then
-      line = normalize(line)
+      line, lastline = normalize(line, lastline)
       if not string.match(line, "^ *$") and not killcheck(line) then
         newlog = newlog .. line .. os_newline
       end
@@ -849,6 +879,7 @@ function formatlualog(logfile, newfile)
     -- Find \discretionary or \whatsit lines:
     -- These may come back later
     if string.match(line, "^%.+\\discretionary$")                or
+       string.match(line, "^%.+\\discretionary %(penalty 50%)$") or
        string.match(line, "^%.+\\discretionary50%|$")            or
        string.match(line, "^%.+\\discretionary50%| replacing $") or
        string.match(line, "^%.+\\whatsit$")                      then
@@ -859,6 +890,10 @@ function formatlualog(logfile, newfile)
     if string.match(line, "^%.+\\mathon$") then
       return line, line
     end
+    -- LuaTeX has a flexible output box
+    line = string.gsub(line,"\\box\\outputbox", "\\box255")
+    -- LuaTeX identifies spaceskip glue
+    line = string.gsub(line,"%(\\spaceskip%) ", " ")
     -- Remove 'display' at end of display math boxes:
     -- LuaTeX omits this as it includes direction in all cases
     line = string.gsub(line, "(\\hbox%(.*), display$", "%1")
@@ -900,9 +935,15 @@ function formatlualog(logfile, newfile)
     local function boxprefix(s)
       return string.gsub(string.match(s, "^(%.+)"), "%.", "%%.")
     end
+    -- 'Recover' some discretionary data
+    if string.match(lastline, "^%.+\\discretionary %(penalty 50%)$") and
+       string.match(line, boxprefix(lastline) .. "%.= ") then
+       return string.gsub(line, "%.= ", ""),""
+    end
     -- Where the last line was a discretionary, looks for the
     -- info one level in about what it represents
     if string.match(lastline, "^%.+\\discretionary$")                or
+       string.match(lastline, "^%.+\\discretionary %(penalty 50%)$") or
        string.match(lastline, "^%.+\\discretionary50%|$")            or
        string.match(lastline, "^%.+\\discretionary50%| replacing $") then
       local prefix = boxprefix(lastline)
@@ -924,6 +965,8 @@ function formatlualog(logfile, newfile)
           if string.match(lastline, "^%.+\\discretionary50%|$") then
             lastline =  string.gsub(lastline, "50%|$", "")
           end
+          -- Remove some info that TeX90 lacks
+          lastline = string.gsub(lastline, " %(penalty 50%)$", "")
           -- A normal (TeX90) discretionary:
           -- add with the line break reintroduced
           return lastline .. os_newline .. line, ""
@@ -1043,8 +1086,8 @@ function listmodules()
   return modules
 end
 
--- Runs a single test: needs the name of the test rather than the .lvt file
--- One 'test' here may apply to multiple engines
+-- Run one test which may have multiple engine-dependent comparisons
+-- Should create a difference file for each failed test
 function runcheck(name, hide)
   local checkengines = checkengines
   if optengines then
@@ -1053,76 +1096,144 @@ function runcheck(name, hide)
   local errorlevel = 0
   for _,i in ipairs(checkengines) do
     -- Allow for luatex == luajittex for .tlg purposes
-    local enginename = i
-    if i == "luajittex" then
-      enginename = "luatex"
-      newfile = testdir .. "/" .. name .. "." .. i .. logext
-    end
-    local testname = name .. "." .. enginename
-    local difffile = testdir .. "/" .. testname .. os_diffext
-    local newfile  = newfile or testdir .. "/" .. testname .. logext
-    -- Use engine-specific file if available
-    local tlgfile  = locate(
-      {testfiledir, unpackdir},
-      {testname .. tlgext, name .. tlgext}
-    )
-    if tlgfile then
-      cp(name .. tlgext, testfiledir, testdir)
-    else
-      -- Attempt to generate missing test goal from expectation
-      tlgfile = testdir .. "/" .. testname .. tlgext
-      if not locate({unpackdir, testfiledir}, {name .. lveext}) then
-        print(
-          "Error: failed to find " .. tlgext .. " or "
-            .. lveext .. " file for " .. name .. "!"
-        )
-        os.exit(1)
-      end
-      runtest(name, i, hide, lveext)
-      ren(testdir, testname .. logext, testname .. tlgext)
-    end
-    runtest(name, i, hide, lvtext)
-    if os_windows then
-      tlgfile = unix_to_win(tlgfile)
+    local engine = i
+    if i == "luajitex" then
+      engine = "luatex"
     end
+    checkpdf = setup_check(name, engine)
+    runtest(name, i, hide, lvtext, checkpdf)
+    -- Generation of results depends on test type
     local errlevel
-    -- Do additional log formatting if the engine is LuaTeX, there is no
-    -- LuaTeX-specific .tlg file and the default engine is not LuaTeX
-    if enginename == "luatex"
-      and not string.match(tlgfile, "%.luatex" .. "%" .. tlgext)
-      and stdengine ~= "luatex"
-      and stdengine ~= "luajittex" then
-      local luatlgfile = testdir .. "/" .. name .. ".luatex" ..  tlgext
-      if os_windows then
-        luatlgfile = unix_to_win(luatlgfile)
-      end
-      formatlualog(tlgfile, luatlgfile)
-      formatlualog(newfile, newfile)
-      errlevel = os.execute(
-        os_diffexe .. " " .. luatlgfile .. " " .. newfile
-          .. " > " .. difffile
-      )
+    if checkpdf then
+      errlevel = compare_pdf(name, engine)
     else
-      errlevel = os.execute(
-        os_diffexe .. " " .. tlgfile .. " " .. newfile .. " > " .. difffile
-      )
+      errlevel = compare_tlg(name, engine)
     end
-    if errlevel == 0 then
-      os.remove(difffile)
-    else
-      if opthalt then
-        checkdiff()
-        return errlevel
-      end
+    if errlevel ~= 0 and opthalt then
+      checkdiff()
+      return errlevel
+    end
+    if errlevel > errorlevel then
       errorlevel = errlevel
     end
   end
   return errorlevel
 end
 
+function setup_check(name, engine)
+  local testname = name .. "." .. engine
+  local pdffile = locate(
+    {testfiledir, unpackdir},
+    {testname .. pdfext, name .. pdfext}
+  )
+  local tlgfile = locate(
+    {testfiledir, unpackdir},
+    {testname .. tlgext, name .. tlgext}
+  )
+  -- Attempt to generate missing reference file from expectation
+  if not (pdffile or tlgfile) then
+    if not locate({unpackdir, testfiledir}, {name .. lveext}) then
+      print(
+        "Error: failed to find " .. pdfext .. ", " .. tlgext .. " or "
+          .. lveext .. " file for " .. name .. "!"
+      )
+      os.exit(1)
+    end
+    runtest(name, engine, true, lveext, true)
+    pdffile = testdir .. "/" .. testname .. pdfext
+    -- If a PDF is generated use it for comparisons
+    if not fileexists(pdffile) then
+      pdffile = nil
+      ren(testdir, testname .. logext, testname .. tlgext)
+    end
+  else
+    -- Install comparison files found
+    for _,v in pairs({pdffile, tlgfile}) do
+      if v then
+        cp(
+          string.match(v, ".*/(.*)"),
+          string.match(v, "(.*)/.*"),
+          testdir
+        )
+      end
+    end
+  end
+  if pdffile then
+    local pdffile = string.match(pdffile, ".*/(.*)")
+    ren(
+      testdir,
+      pdffile,
+      string.gsub(pdffile, pdfext .. "$", ".ref" .. pdfext)
+    )
+    return true
+  else
+    return false
+  end
+end
+
+function compare_pdf(name, engine)
+  local errorlevel
+  local testname = name .. "." .. engine
+  local cmpfile    = testdir .. "/" .. testname .. os_cmpext
+  local pdffile    = testdir .. "/" .. testname .. pdfext
+  local refpdffile = locate(
+    {testdir}, {testname .. ".ref" .. pdfext, name .. ".ref" .. pdfext}
+  )
+  if not refpdffile then
+    return
+  end
+  if os_windows then
+    refpdffile = unix_to_win(refpdffile)
+  end
+  errorlevel = os.execute(
+    os_cmpexe .. " " .. refpdffile .. " " .. pdffile .. " > " .. cmpfile
+  )
+  if errorlevel == 0 then
+    os.remove(cmpfile)
+  end
+  return errorlevel
+end
+
+function compare_tlg(name, engine)
+  local errorlevel
+  local testname = name .. "." .. engine
+  local difffile = testdir .. "/" .. testname .. os_diffext
+  local logfile  = testdir .. "/" .. testname .. logext
+  local tlgfile  = locate({testdir}, {testname .. tlgext, name .. tlgext})
+  if not tlgfile then
+    return
+  end
+  if os_windows then
+    tlgfile = unix_to_win(tlgfile)
+  end
+  -- Do additional log formatting if the engine is LuaTeX, there is no
+  -- LuaTeX-specific .tlg file and the default engine is not LuaTeX
+  if engine == "luatex"
+    and not string.match(tlgfile, "%.luatex" .. "%" .. tlgext)
+    and stdengine ~= "luatex"
+    and stdengine ~= "luajittex"
+    then
+    local luatlgfile = testdir .. "/" .. name .. ".luatex" ..  tlgext
+    if os_windows then
+      luatlgfile = unix_to_win(luatlgfile)
+    end
+    formatlualog(tlgfile, luatlgfile)
+    formatlualog(logfile, logfile)
+    -- This allows code sharing below: we only need the .tlg name in one place
+    tlgfile = luatlgfile
+  end
+  errorlevel = os.execute(
+    os_diffexe .. " " .. tlgfile .. " " .. logfile .. " > " .. difffile
+  )
+  if errorlevel == 0 then
+    os.remove(difffile)
+  end
+  return errorlevel
+end
+
 -- Run one of the test files: doesn't check the result so suitable for
 -- both creating and verifying .tlg files
-function runtest(name, engine, hide, ext)
+function runtest(name, engine, hide, ext, makepdf)
   local lvtfile = name .. (ext or lvtext)
   cp(lvtfile, fileexists(testfiledir .. "/" .. lvtfile)
     and testfiledir or unpackdir, testdir)
@@ -1151,9 +1262,23 @@ function runtest(name, engine, hide, ext)
   end
   -- Special casing for XeTeX engine
   local checkopts = checkopts
-  if string.match(engine, "xetex") then
+  if string.match(engine, "xetex") and not makepdf then
     checkopts = checkopts .. " -no-pdf"
   end
+  -- Special casing for ConTeXt
+  if string.match(checkformat, "^context$") then
+    format = ""
+    if engine == "luatex" or engine == "luajittex" then
+      realengine = "context"
+    elseif engine == "pdftex" then
+      realengine = "texexec"
+    elseif engine == "xetex" then
+      realengine = "texexec --xetex"
+    else
+      print("Engine incompatible with format")
+      os.exit(1)
+    end
+  end
   local logfile = testdir .. "/" .. name .. logext
   local newfile = testdir .. "/" .. name .. "." .. engine .. logext
   local asciiopt = ""
@@ -1173,14 +1298,24 @@ function runtest(name, engine, hide, ext)
       -- Avoid spurious output from (u)pTeX
       os_setenv .. " GUESS_INPUT_KANJI_ENCODING=0"
         .. os_concat ..
+      -- Fix the time of the run
+      os_setenv .. " SOURCE_DATE_EPOCH=" .. epoch
+        .. os_concat ..
+      os_setenv .. " SOURCE_DATE_EPOCH_TEX_PRIMITIVES=1"
+        .. os_concat ..
       -- Ensure lines are of a known length
       os_setenv .. " max_print_line=" .. maxprintline
         .. os_concat ..
       realengine ..  format .. " "
         .. checkopts .. " " .. asciiopt .. lvtfile
         .. (hide and (" > " .. os_null) or "")
+        .. os_concat ..
+      runtest_tasks(stripext(lvtfile))
     )
   end
+  if makepdf and fileexists(testdir .. "/" .. name .. dviext) then
+    dvitopdf(name, testdir, engine, hide)
+  end
   formatlog(logfile, newfile, engine)
   -- Store secondary files for this engine
   for _,i in ipairs(filelist(testdir, name .. ".???")) do
@@ -1197,6 +1332,34 @@ function runtest(name, engine, hide, ext)
   end
 end
 
+-- A hook to allow additional tasks to run for the tests
+runtest_tasks = runtest_tasks or function(name)
+  return ""
+end
+
+function dvitopdf(name, dir, engine, hide)
+  if string.match(engine, "^u?ptex$") then
+    run(
+      dir,
+      os_setenv .. " SOURCE_DATE_EPOCH=" .. epoch
+        .. os_concat ..
+     "dvipdfmx  " .. name .. dviext
+       .. (hide and (" > " .. os_null) or "")
+    )
+  else
+    run(
+      dir,
+      os_setenv .. " SOURCE_DATE_EPOCH=" .. epoch
+        .. os_concat ..
+     "dvips " .. name .. dviext
+       .. (hide and (" > " .. os_null) or "")
+       .. os_concat ..
+     "ps2pdf " .. name .. psext
+        .. (hide and (" > " .. os_null) or "")
+    )
+  end
+end
+
 -- Strip the extension from a file name (if present)
 function stripext(file)
   local name = string.match(file, "^(.*)%.")
@@ -1323,7 +1486,7 @@ typeset = typeset or function(file)
         )
       end
       errorlevel = cycle(name)
-      if errorlevel ~= 0 then
+      if errorlevel == 0 then
         errorlevel = cycle(name)
       end
     end
@@ -1334,7 +1497,7 @@ end
 -- Standard versions of the main targets for building modules
 
 -- Simply print out how to use the build system
-help = help or function()
+function help()
   print("usage: " .. arg[0] .. " <command> [<options>] [<names>]")
   print("")
   print("The most commonly used l3build commands are:")
@@ -1359,8 +1522,11 @@ help = help or function()
   print("   --date|-d           Sets the date to insert into sources")
   print("   --engine|-e         Sets the engine to use for running test")
   print("   --halt-on-error|-H  Stops running tests after the first failure")
-  print("   --version|-v        Sets the version to insert into sources")
+  print("   --pdf|-p            Check/save PDF files")
+  print("   --quiet|-q          Suppresses TeX output when unpacking")
+  print("   --release|-r        Sets the release to insert into sources")
   print("")
+  print("See l3build.pdf for further details.")
 end
 
 function check(names)
@@ -1415,6 +1581,9 @@ function checkdiff()
   for _,i in ipairs(filelist(testdir, "*" .. os_diffext)) do
     print("  - " .. testdir .. "/" .. i)
   end
+  for _,i in ipairs(filelist(testdir, "*" .. os_cmpext)) do
+    print("  - " .. testdir .. "/" .. i)
+  end
   print("")
 end
 
@@ -1498,7 +1667,7 @@ function ctan(standalone)
       dir,
       zipexe .. " " .. zipopts .. " -ll ".. zipname .. " " .. "."
         .. (
-          (binfiles or  exclude) and (" -x" .. binfiles .. " " .. exclude)
+          (binfiles or exclude) and (" -x" .. binfiles .. " " .. exclude)
           or ""
         )
     )
@@ -1662,13 +1831,21 @@ function save(names)
     local engine
     for _,engine in pairs(engines) do
       local tlgengine = ((engine == stdengine and "") or "." .. engine)
-      local tlgfile = name .. tlgengine .. tlgext
-      local newfile = name .. "." .. engine .. logext
+      local tlgfile  = name .. tlgengine .. tlgext
+      local spdffile = name .. tlgengine .. pdfext
+      local newfile  = name .. "." .. engine .. logext
+      local pdffile  = name .. "." .. engine .. pdfext
+      local refext = ((optpdf and pdfext) or tlgext)
       if testexists(name) then
-        print("Creating and copying " .. tlgfile)
-        runtest(name, engine, false, lvtext)
-        ren(testdir, newfile, tlgfile)
-        cp(tlgfile, testdir, testfiledir)
+        print("Creating and copying " .. refext)
+        runtest(name, engine, false, lvtext, optpdf)
+        if optpdf then
+          ren(testdir, pdffile, spdffile)
+          cp(spdffile, testdir, testfiledir)
+        else
+          ren(testdir, newfile, tlgfile)
+          cp(tlgfile, testdir, testfiledir)
+        end
         if fileexists(unpackdir .. "/" .. tlgfile) then
           print(
             "Saved " .. tlgext
@@ -1693,7 +1870,7 @@ end
 -- Provide some standard search-and-replace functions
 if versionform ~= "" and not setversion_update_line then
   if versionform == "ProvidesPackage" then
-    function setversion_update_line(line, date, version)
+    function setversion_update_line(line, date, release)
       -- No real regex so do it one type at a time
       for _,i in pairs({"Class", "File", "Package"}) do
         if string.match(
@@ -1703,7 +1880,7 @@ if versionform ~= "" and not setversion_update_line then
           line = string.gsub(line, "%[%d%d%d%d/%d%d/%d%d", "["
             .. string.gsub(date, "%-", "/"))
           line = string.gsub(
-            line, "(%[%d%d%d%d/%d%d/%d%d) [^ ]*", "%1 " .. version
+            line, "(%[%d%d%d%d/%d%d/%d%d) [^ ]*", "%1 " .. release
           )
           break
         end
@@ -1711,7 +1888,7 @@ if versionform ~= "" and not setversion_update_line then
       return line
     end
   elseif versionform == "ProvidesExplPackage" then
-    function setversion_update_line(line, date, version)
+    function setversion_update_line(line, date, release)
       -- No real regex so do it one type at a time
       for _,i in pairs({"Class", "File", "Package"}) do
         if string.match(
@@ -1721,7 +1898,7 @@ if versionform ~= "" and not setversion_update_line then
           line = string.gsub(
             line,
             "{%d%d%d%d/%d%d/%d%d}( *){[^}]*}",
-            "{" .. string.gsub(date, "%-", "/") .. "}%1{" .. version .. "}"
+            "{" .. string.gsub(date, "%-", "/") .. "}%1{" .. release .. "}"
           )
           break
         end
@@ -1729,22 +1906,22 @@ if versionform ~= "" and not setversion_update_line then
       return line
     end
   elseif versionform == "filename" then
-    function setversion_update_line(line, date, version)
+    function setversion_update_line(line, date, release)
       if string.match(line, "^\\def\\filedate{%d%d%d%d/%d%d/%d%d}$") then
         line = "\\def\\filedate{" .. string.gsub(date, "%-", "/") .. "}"
       end
       if string.match(line, "^\\def\\fileversion{[^}]+}$") then
-        line = "\\def\\fileversion{" .. version .. "}"
+        line = "\\def\\fileversion{" .. release .. "}"
       end
       return line
     end
   elseif versionform == "ExplFileName" then
-    function setversion_update_line(line, date, version)
+    function setversion_update_line(line, date, release)
       if string.match(line, "^\\def\\ExplFileDate{%d%d%d%d/%d%d/%d%d}$") then
         line = "\\def\\ExplFileDate{" .. string.gsub(date, "%-", "/") .. "}"
       end
       if string.match(line, "^\\def\\ExplFileVersion{[^}]+}$") then
-        line = "\\def\\ExplFileVersion{" .. version .. "}"
+        line = "\\def\\ExplFileVersion{" .. release .. "}"
       end
       return line
     end
@@ -1752,7 +1929,7 @@ if versionform ~= "" and not setversion_update_line then
 end
 
 -- Used to actually carry out search-and-replace
-setversion_update_line = setversion_update_line or function(line, date, version)
+setversion_update_line = setversion_update_line or function(line, date, release)
   return line
 end
 
@@ -1789,13 +1966,13 @@ function setversion()
   if optdate then
     date = optdate[1] or date
   end
-  local version = -1
-  if optversion then
-    version = optversion[1] or version
+  local release = -1
+  if optrelease then
+    release = optrelease[1] or release
   end
   for _,i in pairs(versionfiles) do
     for _,j in pairs(filelist(".", i)) do
-      rewrite(j, date, version)
+      rewrite(j, date, release)
     end
   end
   return 0
@@ -1929,7 +2106,7 @@ function stdmain(target, files)
       errorlevel = bundlectan()
     elseif target == "doc" then
       errorlevel = doc(files)
-    elseif target == "check" and testfiledir ~= "" then
+    elseif target == "check" then
       errorlevel = check(files)
     elseif target == "clean" then
       errorlevel = clean()
@@ -1939,7 +2116,7 @@ function stdmain(target, files)
       errorlevel = ctan(true)
     elseif target == "install" then
       errorlevel = install()
-    elseif target == "save" and testfiledir ~= "" then
+    elseif target == "save" then
       if next(files) then
         errorlevel = save(files)
       else
